/**
 * @brief           bdpv management
 * @file            bdpv.c
 * @author          <a href="">Dom-IP</a>
 * @dependencies    none
 * @compiler        MPLAB C18 v2.10 or higher <br>
 *                  HITECH PICC-18 V8.35PL3 or higher
 *
 * @section description Description
 **********************************
 * This module contains functions used for the leds management of evaluation boards
 *
 *********************************************************************/

 /*********************************************************************
 * Software License Agreement
 *
 * The software supplied herewith is owned by Benoit Sandre, and is
 * protected under applicable copyright laws.  The code may
 * be modified and can be used free of charge for commercial and non commercial
 * applications. All rights are reserved. Any use in violation of the foregoing
 * restrictions may subject the user to criminal sanctions under applicable laws,
 * as well as to civil liability for the breach of the terms and conditions of this license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN 'AS IS' CONDITION. NO WARRANTIES, WHETHER EXPRESS,
 * IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE
 * COMPANY SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 **********************************************************************
 * File History
 *
 * 2012-01-23, Dom-IP
 *    - Initial version
 *********************************************************************/
#include <string.h>
#include "projdefs.h"
#include "appcfg.h"
#include "net\tick.h"
#include "net\compiler.h"
#include "leds.h"
#include "net\dns.h"
#include "net\httpclient.h"
#include "net\hashes.h"
#include "proto_board\teleinfo.h"
#include "sntp\sntp.h"
#include "bdpv.h"


////////////////////////////////
//BDPV application variables
////////////////////////////////

enum BDPV_STATUS BDPVStatus=BDPV_STANDBY;

DWORD nextBdpvUpdate=0; // time of the next update
DWORD lastBdpvUpdate=0; // time of the last update
char adco[13]; //numro de srie du compteur edf
char index[10]; //index du compteur edf
char bdpvTime[13]; //date d'acquisition de l'index
BYTE api_sig[16]={0}; //signature
BYTE BdpvHttpResp[64]={0}; //response de la requete sur le site bdpv

HASH_SUM md5_hash;

//BDPV domaine name
rom char bdpvDomain[]="www.bdpv.fr";

//teleinfo tags values
const rom BYTE adcoTag[]="\nADCO";
const rom BYTE indexTag[]="\nBASE";

//checksum  values
const rom BYTE STR_ADCO[]="ADCO";
const rom BYTE STR_DATE[]="DATE";
const rom BYTE STR_INDEX[]="INDEX";
const rom BYTE STR_api_demandeur[]="api_demandeur";



void setBdpvTimer(void) {
DWORD day,sec;
	//get day number since 1970
	day=(SNTPGetUTCSeconds() / SECS_DAY);
	
	//get sec number since midnight
	sec= SNTPGetUTCSeconds() % SECS_DAY;
	
	//add a day and convert the value in seconds and save it
	//add sec number since midnight % 2 hours
	nextBdpvUpdate=(day +1) * SECS_DAY + (sec %  (2*SECS_HOUR));
}

//bdpv application initialization
void appcfgBdpv() {
DWORD temp;

//if the login is not null
if(appcfgGetc(APPCFG_BDPV_APIDEM0) !=0) {
	//launch task if not running
	if (BDPVStatus==BDPV_STANDBY) BDPVStatus=BDPV_GET_DATA;
}

}

void getYYYYMMDDHHMM(DWORD *timeStamp,char *result) {
struct tm temp;
	gmtime(timeStamp,&temp);
	
	//day
	if (temp.tm_mday <=9) {
		result[0]='0';
		itoa(temp.tm_mday, &result[1]);
	} else itoa(temp.tm_mday, &result[0]);
	
	//month
	if (temp.tm_mon <=8) {
		result[2]='0';
		itoa(temp.tm_mon+1, &result[3]);
	} else itoa(temp.tm_mon+1, &result[2]);
	
	//year
	itoa(temp.tm_year+1900,&result[4]);
	
	//hour
	if (temp.tm_hour <=9) {
		result[8]='0';
		itoa(temp.tm_hour, &result[9]);
	} else itoa(temp.tm_hour, &result[8]);
	
	//minute
	if (temp.tm_min <=9) {
		result[10]='0';
		itoa(temp.tm_min, &result[11]);
	} else itoa(temp.tm_min, &result[10]);

}


//bdpv application processing
void appBdpv() {
static char HostName[32];
static char filename[9];
static IP_ADDR HostIP;
static TCP_FILESEND_INFO Req_Http;

char strTemp[16];
BYTE i,j;

//if the status is null, check for a new transmission
if (BDPVStatus==BDPV_STANDBY) {
	// if the login is not empty
	if(appcfgGetc(APPCFG_BDPV_APIDEM0) !=0) {
			//launch transmission
			if (SNTPGetUTCSeconds() >= nextBdpvUpdate) {
				//update task state
				BDPVStatus=BDPV_GET_DATA;
			}
	}
}
//If status ==1 extract datas
else if (BDPVStatus==BDPV_GET_DATA) {
		//save time
		lastBdpvUpdate=SNTPGetUTCSeconds();	
		//clear http request resul
		BdpvHttpResp[0]=0;
		//extract ADCO value
		getTeleinfoField(trameTeleInfo1,adcoTag,12,adco);
		//extract INDEX value
		getTeleinfoField(trameTeleInfo1,indexTag,9,index);
		//format DATE
		getYYYYMMDDHHMM(&lastBdpvUpdate,bdpvTime);
		BDPVStatus=BDPV_MD5;
}
//If status =2 calculate md5
else if (BDPVStatus==BDPV_MD5) {
		//calculate MD5
		MD5Initialize(&md5_hash);
		//add password
			//Get password from EEPROM
			for(i=0;i<16;i++) {
			strTemp[i]=appcfgGetc(APPCFG_BDPV_APIPASS0+i);
			}
			//add it
			MD5AddData(&md5_hash,(BYTE *)strTemp,strlen(strTemp));
		//add 'ADCO' const
		MD5AddROMData(&md5_hash,(rom BYTE*)STR_ADCO,4);
		//add adco value
		MD5AddData(&md5_hash,(BYTE *)adco,12);
		//add 'DATE' const
		MD5AddROMData(&md5_hash,(rom BYTE*)STR_DATE,4);
		//add DATE value
		MD5AddData(&md5_hash,(BYTE *)bdpvTime,12);
		//add 'INDEX' const
		MD5AddROMData(&md5_hash,(rom BYTE*)STR_INDEX,5);
		//add  index value
		MD5AddData(&md5_hash,(BYTE *)index,9);
		//add 'api_demandeur' const
		MD5AddROMData(&md5_hash,(rom BYTE*)STR_api_demandeur,13);
		//add api_demandeur value
			//Get api_demandeur from EEPROM
			for(i=0;i<16;i++) {
			strTemp[i]=appcfgGetc(APPCFG_BDPV_APIDEM0+i);
			}
			//add it
			MD5AddData(&md5_hash,(BYTE *)strTemp,strlen(strTemp));
		//calculate md5
		MD5Calculate(&md5_hash,&api_sig[0]);
		BDPVStatus=BDPV_DNS_INI;
}
// if status=3, DNS Resolution init
else if (BDPVStatus==BDPV_DNS_INI) {
	//Get host name from ROM
	strcpypgm2ram(HostName,bdpvDomain);
	//launch resolution
	DNSResolve((BYTE *)HostName);
	//Status update
	BDPVStatus=BDPV_DNS;
}
//Waiting for DNS resolution
else if (BDPVStatus==BDPV_DNS) {
	//if dns request is ended
	if ( DNSIsResolved(&Req_Http.serverNode.IPAddr)) {
		BDPVStatus=BDPV_HTTP_INI;
	}
}
//Init HTTP request
else if (BDPVStatus==BDPV_HTTP_INI) {
	//Initialize http request
	initHttpClient(&Req_Http,HTTP_SERVICE_PORT);
	
	//The request data is saved in the file "requete.htp"
	strcpypgm2ram(filename,(rom char *)"bdpvreq.cgi");
	BDPVStatus=BDPV_HTTP_REQ;
}
//Continue HTTP request
else if (BDPVStatus==BDPV_HTTP_REQ) {
	//start or continue http request
	i=sendFileToTcpServer(filename,&Req_Http);

	//if http request is terminated
	if (i!=0xff) {
		//if http request has been send successfully
		if (i==0) BDPVStatus=BDPV_HTTP_RES;
		//if an error occur, 
		else {
			BDPVStatus=BDPV_STANDBY;
			setBdpvTimer();
		}
	}
}
//Get HTTP result line
else if (BDPVStatus==BDPV_HTTP_RES) {
	//wait for data on tcp socket
	j=0;
	i=getHttpResponseLine(&Req_Http,BdpvHttpResp,sizeof(BdpvHttpResp),&j);
	i=skipHttpHeaders(&Req_Http);
	i=getHttpResponseLine(&Req_Http,BdpvHttpResp,sizeof(BdpvHttpResp),&j);
	//if line has been read
	if (i==0) {
		closeTcp(&Req_Http);
		BDPVStatus=BDPV_STANDBY;
		setBdpvTimer();
	}
	//if an error occur
	else if (i!=TCPU_NOT_FINISHED) {
		BDPVStatus=BDPV_STANDBY;
		//plan next upload
		setBdpvTimer();
	}

}
}


