/**
 * @brief           teleinfo reception
 * @file            teleinfo.h
 * @author          <a href="">Benot Sandr</a>
 * @dependencies    none
 * @compiler        MPLAB C18 v2.10 or higher <br>
 *                  HITECH PICC-18 V8.35PL3 or higher
 *
 * @section description Description
 **********************************
 * This module contains functions used to get teleinformation frame from an EDF meter
 *
 *********************************************************************/

 /*********************************************************************
 * Software License Agreement
 *
 * according GPL V2.0
 **********************************************************************
 * File History
 *
 * 2009-06-16, Benoit Sandr (DH):
 *    - Initial version
 *********************************************************************/

#ifndef TELEINFO_H
#define TELEINFO_H

#define TAILLE_MAX_TRAME_TELEINFO 256

extern BYTE trameTeleInfo1[];
extern BYTE trameTeleInfo2[];
extern BYTE *teleinfoBuffer;
extern BYTE statusTeleInfo;
extern TICK16 startTimer; 



/* TeleInfo process state definition */

#define TELEINFO_IDLE 0u
#define TELEINFO_POWER_ON 1u
#define TELEINFO_START_WAITING 2u
#define TELEINFO_RECEIVING 3u
#define TELEINFO_RX_OK 4u
#define TELEINFO_RX_KO 5u

/* Teleinfo Timer . Unit is 10 ms*/
#define TELEINFO_POWER_ON_TIMER 100  // Rx will start xx * 10 ms  after interface power on
#define TELEINFO_TIMEOUT 300 // Rx is stopped if no start is detected before xx * 10 ms

/* init Teleinfo Power PIN*/
#define INIT_TELEINFO 	LATB1=0;TRISB_RB1=0;LATB2=0;TRISB_RB2=0;statusTeleInfo=TELEINFO_IDLE;

#define POWER_ON_TI1 LATB1=1;
#define POWER_ON_TI2 LATB2=1;

#define POWER_OFF_TI LATB1=0;LATB2=0;

/* Launch TELEINFO1 RX*/
#define START_TELEINFO_RX1 POWER_ON_TI1;teleinfoBuffer=trameTeleInfo1;statusTeleInfo=TELEINFO_POWER_ON;startTimer=TickGet16bit();
/* Launch TELEINFO2 RX*/
#define START_TELEINFO_RX2 POWER_ON_TI2;teleinfoBuffer=trameTeleInfo2;statusTeleInfo=TELEINFO_POWER_ON;startTimer=TickGet16bit();

/**
 * Run TeleInfo Rx task
 *
 * @param delay     Delay in seconds
 */
BYTE processTeleInfo(void);

/**
 * extract value from the teleinfo frame
 *
 * @param teleInfoFrame, tag to be search (begininning with /n, field length and output buffer
 * output BYTE :  0 if successfull, 1 in case of error
 */
BYTE getTeleinfoField(BYTE* frame,const rom BYTE* tag,BYTE fieldLength,char* value);

#endif
