/**
 * @brief           leds management
 * @file            websampling.c
 * @author          <a href="">Dom-IP</a>
 * @dependencies    none
 * @compiler        MPLAB C18 v2.10 or higher <br>
 *                  HITECH PICC-18 V8.35PL3 or higher
 *
 * @section description Description
 **********************************
 * This module contains functions used for the leds management of evaluation boards
 *
 *********************************************************************/

 /*********************************************************************
 * Software License Agreement
 *
 * The software supplied herewith is owned by Benoit Sandre, and is
 * protected under applicable copyright laws.  The code may
 * be modified and can be used free of charge for commercial and non commercial
 * applications. All rights are reserved. Any use in violation of the foregoing
 * restrictions may subject the user to criminal sanctions under applicable laws,
 * as well as to civil liability for the breach of the terms and conditions of this license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN 'AS IS' CONDITION. NO WARRANTIES, WHETHER EXPRESS,
 * IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE
 * COMPANY SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 **********************************************************************
 * File History
 *
 * 2009-06-01, Dom-IP
 *    - Initial version
 *********************************************************************/
#include <string.h>
#include "projdefs.h"
#include "appcfg.h"
#include "net\tick.h"
#include "net\compiler.h"
#include "leds.h"
#include "net\dns.h"
#include "net\httpclient.h"
#include "teleinfo.h"
#include "proto_board\leds.h"
#include "websampling.h"

/////////////////////////////////////////////////
//Function prototypes
//NA

////////////////////////////////
//Websampling application variables
////////////////////////////////

TICK16 tmrSec;
unsigned short nameResolutionCount=0;

WORD httpBodyLength;
BYTE WsHttpResp[64]={0}; //http answser string


enum WS_STATUS WSstatus=0;

//WebSampling application initialization
void appcfgWebSampling() {
	//Init Teleinfo
	INIT_TELEINFO;
	//init Timer
	tmrSec=TickGet16bitSec()-((TICK16) WS_INTERVAL );
	//force  dns request
	nameResolutionCount=0;
}

//WebSampling application processing
void appWebSampling() {
static BYTE HostName[32];
static char filename[9];
static IP_ADDR HostIP;
static TCP_FILESEND_INFO Req_Http;

BYTE i,j;
		//In standby state
		if (WSstatus==WS_STANDBY) {
		//Every 60 seconds, 
			SetLeds(STANDBY_LED);
			if (  TickGetDiffSec(tmrSec) >= ((TICK16) WS_INTERVAL ) ) {
				//reload the seconds timer
				tmrSec+=((TICK16)60);
				//Relaunch send if task is in Standby
				WSstatus=WS_DNS_INI;
				ClearLeds(STANDBY_LED);
			}
		}
		// DNS Resolution init
		else if (WSstatus==WS_DNS_INI) {
			// Si la rsolution DNS doit tre lance
			if (nameResolutionCount--== 0u ) {
				//Reinit Tmr
				nameResolutionCount=NAME_RESOLUTION_MAX_COUNT;
				
				//Get host name from EEPROM
				for(i=0;i< 32u;i++) {
					HostName[i]=appcfgGetc(APPCFG_HOST_NAME0+i);
				}
				//string max length is 32 chars
				HostName[31]=0;
				
				// If hostname is defined
				if ((BYTE)HostName[0]!=0u) {
					//Launch DNS request
					DNSResolve(HostName);
					//Status update

					WSstatus=WS_DNS;
				}
				// Otherwise launch teleinfo frame receiving 
				else WSstatus=WS_TI1_INI;
			}
			// Otherwise launch teleinfo 1 frame receiving 
			else WSstatus=WS_TI1_INI;
		}
		//Waiting for DNS resolution
		else if (WSstatus==WS_DNS) {
			//if dns request is ended
			if ( DNSIsResolved(&HostIP)) {
				//Save result in EEPROM
				for(i=0;i<4u;i++) {
					appcfgPutc(APPCFG_HOST_IP0+i, HostIP.v[i]);
				}
				//Launch teleinfo 1 frame receiving 
				WSstatus=WS_TI1_INI;
			}
		}
		//init teleinformation 1  frame reception
		else if (WSstatus==WS_TI1_INI)  {
			SetLeds(TELEINFO1_LED);
			//start teleinfo frame 1 reception
			START_TELEINFO_RX1;
			WSstatus=WS_TI1;
		}
		//teleinformation frame reception
		else if (WSstatus==WS_TI1) {
			//if teleinfo reception is ended, launch sending of http request
			i=processTeleInfo();
			//if frame has been successfully received
			if (i==TELEINFO_RX_OK) {
				//switch led off
				ClearLeds(TELEINFO1_LED);
				WSstatus=WS_TI2_INI;
			}
			//if an error occur led stay on
			if (i==TELEINFO_RX_KO) WSstatus=WS_TI2_INI;
		}
		//init teleinformation 2  frame reception
		else if (WSstatus==WS_TI2_INI)  {
			SetLeds(TELEINFO2_LED);
			//start teleinfo frame 2 reception
			START_TELEINFO_RX2;
			WSstatus=WS_TI;
		}
		//teleinformation 2 frame reception
		else if (WSstatus==WS_TI) {
			//if teleinfo reception is ended, launch sending of http request
			i=processTeleInfo();
			//if frame has been successfully received
			if (i==TELEINFO_RX_OK) {
				//switch led off
				ClearLeds(TELEINFO2_LED);			
				WSstatus=WS_HTTP_INI;
			}
			//if an error occur led stay on
			if (i==TELEINFO_RX_KO) WSstatus=WS_HTTP_INI;
		}
		//Init HTTP request
		else if (WSstatus==WS_HTTP_INI) {	
			//set led
			SetLeds(HTTP_LED);
			
			//Initialize http request
			initHttpClientIpEeprom(&Req_Http,APPCFG_HOST_IP0,HTTP_SERVICE_PORT);
			//calculate body lenght of the request
			httpBodyLength=147+strlen(trameTeleInfo1)+strlen(trameTeleInfo2);
			
			//The request data is saved in the file "requete.htp"
			strcpypgm2ram(filename,(char*) "httpreq.cgi");
			WSstatus=WS_HTTP_REQ;
		}
		//Continue HTTP request
		else if (WSstatus==WS_HTTP_REQ) {
			//start or continue http request
			i=sendFileToTcpServer(filename,&Req_Http);
			
			//http request has been send successfully
			if (i==0u) WSstatus=WS_HTTP_RES;
			//if an error occur, 
			else if (i!=0xff) {
				WSstatus=WS_STANDBY;
			}	
				
		}
		//get HTTP status line
		else if (WSstatus==WS_HTTP_RES) {
			//wait for data on tcp socket
			j=0;
			i=getHttpResponseLine(&Req_Http,WsHttpResp,sizeof(WsHttpResp),&j);
			if (i==0u) i=skipHttpHeaders(&Req_Http);
			if (i==0u) i=getHttpResponseLine(&Req_Http,WsHttpResp,sizeof(WsHttpResp),&j);			
			//if line has been read
			if (i==0u) {
				closeTcp(&Req_Http);
				WSstatus=WS_STANDBY;
				
				//clear Led
				ClearLeds(HTTP_LED);
			}
			//if an error occur
			else if (i!=TCPU_NOT_FINISHED) {
				WSstatus=WS_STANDBY;
			}
		}

}



